@@ -1,7 +1,7 @@
 import typing
 # ******************************************************************************
 # ******************************************************************************
-def n0isnumeric(value: str) -> bool:
+def isnumber(value: str, max_len: typing.Union[int, None] = None) -> bool:
     if isinstance(value, (int, float)):
         return True
     if not isinstance(value, str):
@@ -9,11 +9,13 @@ def n0isnumeric(value: str) -> bool:
     value = value.strip()
     if value.startswith('+') or value.startswith('-'):
         value = value[1:].strip()
-    if len(value) > 10:
+    if max_len and len(value) > max_len:
         return False
     if value.count('.') == 1:
         value = value.replace('.','0')
     return value.isnumeric()
+def n0isnumeric(value: str) -> bool:
+    return isnumber(value, 10)
 # ******************************************************************************
 def get_key_by_value(dict_: dict, value_: typing.Any):
     """
@@ -221,12 +223,12 @@ def validate_str(
     '''
         value = ""
         validate_str(value) == ""
-        value = "C:\TMP"
-        validate_path(value) == "C:\TMP"
+        value = "C:\\TMP"
+        validate_path(value) == "C:\\TMP"
         value = ""
-        validate_path(value, default_value = "C:\ETC") == "C:\ETC"
+        validate_path(value, default_value = "C:\\ETC") == "C:\\ETC"
         value = "."
-        validate_path(value, default_value = "C:\ETC") == ""
+        validate_path(value, default_value = "C:\\ETC") == ""
     '''
     return default_value if not value else process_str(str)
 # ******************************************************************************
@@ -238,12 +240,12 @@ def validate_path(
     '''
         value = ""
         validate_path(value) == ""
-        value = "C:\TMP"
-        validate_path(value) == "C:\TMP"
+        value = "C:\\TMP"
+        validate_path(value) == "C:\\TMP"
         value = ""
-        validate_path(value, default_value = "C:\ETC") == "C:\ETC"
+        validate_path(value, default_value = "C:\\ETC") == "C:\\ETC"
         value = "."
-        validate_path(value, default_value = "C:\ETC") == ""
+        validate_path(value, default_value = "C:\\ETC") == ""
     '''
     if not value:
         return default_value
@@ -251,28 +253,9 @@ def validate_path(
         return ""
     return process_str(value)
 # ******************************************************************************
-def validate_and_map(
-                    value,
-                    variants: dict,
-                    default_value: str = "",
-) -> str:
-    '''
-        value = "A"
-        validate_and_map(value.upper(), {"A":1, "B":2}, 3) == 1
-        value = "B"
-        validate_and_map(value.upper(), {"A":1, "B":2}, 3) == 2
-        value = "C"
-        validate_and_map(value.upper(), {"A":1, "B":2}, 3) == 3
-    '''
-    if not value:
-        return default_value
-    if value in variants:
-        return variants[upper_value]
-    return default_value
-# ******************************************************************************
 def validate_bool(
                     value,
-                    unknown_value_is : False
+                    unknown_value_is: bool = False
 ):
     '''
         value = "TRUE"
@@ -286,26 +269,22 @@ def validate_bool(
         value = "UNKNOWN"
         validate_bool(value) == False
     '''
-    return validate_and_map(
-                            value.upper() if isinstance(value, str) else value,
-                            {
-                                "TRUE":     True,
-                                "T":        True,
-                                "YES":      True,
-                                "Y":        True,
-                                "1":        True,
-                                True:       True,
-                                1:          True,
-                                "FALSE":    False,
-                                "F":        False,
-                                "NO":       False,
-                                "N":        False,
-                                "0":        False,
-                                False:      False,
-                                0:          False,
-                            },
-                            unknown_value_is
-    )
+    return {
+        "TRUE":     True,
+        "T":        True,
+        "YES":      True,
+        "Y":        True,
+        "1":        True,
+        True:       True,
+        1:          True,
+        "FALSE":    False,
+        "F":        False,
+        "NO":       False,
+        "N":        False,
+        "0":        False,
+        False:      False,
+        0:          False,
+    }.get(value.upper() if isinstance(value, str) else value, unknown_value_is),
 # ******************************************************************************
 def validate_values(value: str,
                     possible_values_the_last_is_default: typing.Union[list, tuple],
@@ -313,11 +292,11 @@ def validate_values(value: str,
 ):
     '''
         value = "A"
-        validate_and_map(value.upper(), ("A", "B", "C")) == "A"
-        value = "B"
-        validate_and_map(value.upper(), ("A", "B", "C")) == "B"
+        validate_values(value.upper(), ("A", "B", "C")) == "A"
+        value = "c"
+        validate_values(value.upper(), ("A", "B", "C")) == "C"
         value = "D"
-        validate_and_map(value.upper(), ("A", "B", "C")) == "C"
+        validate_values(value.upper(), ("A", "B", "C")) == "C"
     '''
     if not isinstance(possible_values_the_last_is_default, (list, tuple)) or not possible_values_the_last_is_default:
         return default_value
